//****************************************************************************
//	GrayDib.cpp implementation file
//

//****************************************************************************
//	Includes

#include "stdafx.h"
#include <math.h>
#include <vfw.h>
#include "GrayDib.h"
//#include	"error.h"

// extern int SumMemGA;

//****************************************************************************
//	CGrayDib constructor

CGrayDib::CGrayDib( void )
{
	m_dwWidth = 0;
	m_dwHeight = 0;
	m_dwRowLength = 0;
	m_hDib = NULL;
}
//****************************************************************************
//	CGrayDib destructor

CGrayDib::~CGrayDib( void )
{
	if ( m_hDib )
	{
//		SumMemGA -= GlobalSize( m_hDib );
		::GlobalFree( m_hDib );
		m_hDib = NULL;
	}
}
//****************************************************************************
//	CGrayDib class
//			GetHandle()

void CGrayDib::operator=( CGrayDib dDib )
{
	m_dwWidth = dDib.m_dwWidth;
	m_dwHeight = dDib.m_dwHeight;
	m_dwRowLength = dDib.m_dwRowLength;
	m_dwDibLength = dDib.m_dwDibLength;

	CopyHandle( dDib.GetHandle());
}
//****************************************************************************
//	CGrayDib class
//			GetHandle()

HGLOBAL	CGrayDib::GetHandle()
{
	return	m_hDib;
}
//****************************************************************************
//	CGrayDib class
//			SetHandle()

BOOL CGrayDib::ReplaceHandle( HGLOBAL hHandle )
{
	if ( m_hDib )
	{
//		SumMemGA -= GlobalSize( m_hDib );
		GlobalFree( m_hDib );
		m_hDib = NULL;
	}

	m_hDib = hHandle;

	PBYTE	pDib;
	if ( !( pDib = ( PBYTE )::GlobalLock( m_hDib )))
		return FALSE;

	m_dwWidth = (( BITMAPINFOHEADER* )pDib )->biWidth;
	m_dwHeight = (( BITMAPINFOHEADER* )pDib )->biHeight;
	m_dwRowLength = AlignToDword( m_dwWidth );
	m_dwDibLength = m_dwRowLength * m_dwHeight;

	::GlobalUnlock( m_hDib );

	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			CopyHandle()

BOOL CGrayDib::CopyHandle( HGLOBAL hHandle )
{
	if ( hHandle == NULL )
		return FALSE;

	if ( m_hDib )
	{
//		SumMemGA -= GlobalSize( m_hDib );
		GlobalFree( m_hDib );
		m_hDib = NULL;
	}

	DWORD dwLength = GlobalSize( hHandle );
	m_hDib = GlobalAlloc( GHND, dwLength );
//	SumMemGA += dwLength;

	if ( m_hDib != NULL )
	{
		void* pDib = GlobalLock( m_hDib );
		void* pHandle = GlobalLock( hHandle );

		memcpy( pDib, pHandle, dwLength );

		GlobalUnlock( m_hDib );
		GlobalUnlock( hHandle );
	}
	else
		return FALSE;

	PBYTE	pDib;
	if ( !( pDib = ( PBYTE )::GlobalLock( m_hDib )))
		return FALSE;

	m_dwWidth = (( BITMAPINFOHEADER* )pDib )->biWidth;
	m_dwHeight = (( BITMAPINFOHEADER* )pDib )->biHeight;
	m_dwRowLength = AlignToDword( m_dwWidth );
	m_dwDibLength = m_dwRowLength * m_dwHeight;

	::GlobalUnlock( m_hDib );
		
	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			CopyHGlobal()

HGLOBAL CGrayDib::CopyHGlobal( HGLOBAL hHandle )
{
	if ( hHandle == NULL )
		return NULL;

	DWORD dwLength = GlobalSize( hHandle );
	HGLOBAL hNew = GlobalAlloc( GHND, dwLength );
//	SumMemGA += dwLength;

	if ( hNew != NULL )
	{
		void* pDib = GlobalLock( hNew );
		void* pHandle = GlobalLock( hHandle );

		memcpy( pDib, pHandle, dwLength );

		GlobalUnlock( hNew );
		GlobalUnlock( hHandle );

		return hNew;
	}

	return NULL;
}

//****************************************************************************
//	CGrayDib class
//			Create( HGLOBAL hHandle )

BOOL CGrayDib::Revert()
{
	PBYTE	pBits;
	if ( !( pBits = LockBits()))
		return FALSE;

	PBYTE	pRevertBits = new BYTE[ m_dwRowLength * m_dwHeight ];
	if (pRevertBits == NULL) 
		return FALSE;

	for ( DWORD i = 0; i < m_dwHeight; i ++ )
		memcpy( &pRevertBits[( m_dwHeight - i - 1 ) * m_dwRowLength ], &pBits[ i * m_dwRowLength ], m_dwRowLength );

	memcpy( pBits, pRevertBits, m_dwDibLength );

	delete pRevertBits;

	if ( !UnlockBits())
		return FALSE;

	return TRUE;
}

//****************************************************************************
//	CGrayDib class
//			Create( HGLOBAL hHandle )

BOOL CGrayDib::Create( HGLOBAL hHandle )
{
	if ( m_hDib )
	{
//		SumMemGA -= GlobalSize( m_hDib );
		::GlobalFree( m_hDib );
		m_hDib = NULL;
	}

	m_hDib = hHandle;

	PBYTE	pDib;
	if ( !( pDib = ( PBYTE )::GlobalLock( m_hDib )))
		return FALSE;

	m_dwWidth = (( BITMAPINFOHEADER* )pDib )->biWidth;
	m_dwHeight = (( BITMAPINFOHEADER* )pDib )->biHeight;
	m_dwRowLength = AlignToDword( m_dwWidth * 3 );
	m_dwDibLength = m_dwRowLength * m_dwHeight;

	::GlobalUnlock( m_hDib );

	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			Create( CSize szSize )

BOOL CGrayDib::Create( CSize szSize )
{
	if ( m_hDib )
	{
//		SumMemGA -= GlobalSize( m_hDib );
		::GlobalFree( m_hDib );
		m_hDib = NULL;
	}

	m_dwWidth = szSize.cx;
	m_dwHeight = szSize.cy;
	m_dwRowLength = AlignToDword( szSize.cx );
	m_dwDibLength = m_dwRowLength * m_dwHeight;

	if ( !( m_hDib = ::GlobalAlloc( GHND, m_dwDibLength 
		+ sizeof( BITMAPINFOHEADER ) + 1024 )))
	{
	//	SumMemGA += m_dwDibLength + sizeof( BITMAPINFOHEADER ) + 1024;
		return FALSE;
	}

	PBYTE	pDib;
	if ( !( pDib = ( PBYTE )::GlobalLock( m_hDib )))
		return FALSE;

	(( BITMAPINFOHEADER* )pDib )->biSize = sizeof( BITMAPINFOHEADER );
	(( BITMAPINFOHEADER* )pDib )->biWidth = m_dwWidth;
	(( BITMAPINFOHEADER* )pDib )->biHeight = m_dwHeight;
	(( BITMAPINFOHEADER* )pDib )->biPlanes = 1;
	(( BITMAPINFOHEADER* )pDib )->biBitCount = 8;
	(( BITMAPINFOHEADER* )pDib )->biCompression = BI_RGB;
	(( BITMAPINFOHEADER* )pDib )->biSizeImage = 0;
	(( BITMAPINFOHEADER* )pDib )->biXPelsPerMeter = 0;
	(( BITMAPINFOHEADER* )pDib )->biYPelsPerMeter = 0;
	(( BITMAPINFOHEADER* )pDib )->biClrUsed = 0;
	(( BITMAPINFOHEADER* )pDib )->biClrImportant = 0;

	::GlobalUnlock( m_hDib );

	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			Create( CSize szSize, BYTE clrColor )

BOOL CGrayDib::Create( CSize szSize, BYTE clrColor )
{
	if ( m_hDib )
	{
//		SumMemGA -= GlobalSize( m_hDib );
		::GlobalFree( m_hDib );
		m_hDib = NULL;
	}

	m_dwWidth = szSize.cx;
	m_dwHeight = szSize.cy;
	m_dwRowLength = AlignToDword( szSize.cx );
	m_dwDibLength = m_dwRowLength * m_dwHeight;

	if ( !( m_hDib = ::GlobalAlloc( GHND, m_dwDibLength 
		+ sizeof( BITMAPINFOHEADER ) + 1024 )))
	{
	//	SumMemGA += m_dwDibLength + sizeof( BITMAPINFOHEADER ) + 1024;
		return FALSE;
	}

	PBYTE	pDib;
	if ( !( pDib = ( PBYTE )::GlobalLock( m_hDib )))
		return FALSE;

	(( BITMAPINFOHEADER* )pDib )->biSize = sizeof( BITMAPINFOHEADER );
	(( BITMAPINFOHEADER* )pDib )->biWidth = m_dwWidth;
	(( BITMAPINFOHEADER* )pDib )->biHeight = m_dwHeight;
	(( BITMAPINFOHEADER* )pDib )->biPlanes = 1;
	(( BITMAPINFOHEADER* )pDib )->biBitCount = 8;
	(( BITMAPINFOHEADER* )pDib )->biCompression = BI_RGB;
	(( BITMAPINFOHEADER* )pDib )->biSizeImage = 0;
	(( BITMAPINFOHEADER* )pDib )->biXPelsPerMeter = 0;
	(( BITMAPINFOHEADER* )pDib )->biYPelsPerMeter = 0;
	(( BITMAPINFOHEADER* )pDib )->biClrUsed = 0;
	(( BITMAPINFOHEADER* )pDib )->biClrImportant = 0;

	::GlobalUnlock( m_hDib );

	if ( !Fill( clrColor ))
		return FALSE;

	SetGrayPalette ();

	return TRUE;
}

//****************************************************************************
//	CGrayDib class
//			Create( BYTE *pBuffer, int sizeX, int sizeY )

BOOL CGrayDib::Create( BYTE *pBuffer, int sizeX, int sizeY )
{
	if ( m_hDib )
	{
//		SumMemGA -= GlobalSize( m_hDib );
		::GlobalFree( m_hDib );
		m_hDib = NULL;
	}

	m_dwWidth = sizeX;
	m_dwHeight = sizeY;
	m_dwRowLength = AlignToDword( sizeX );
	m_dwDibLength = m_dwRowLength * m_dwHeight;

	if ( !( m_hDib = ::GlobalAlloc( GHND, m_dwDibLength 
		+ sizeof( BITMAPINFOHEADER ) + 1024 )))
	{
	//	SumMemGA += m_dwDibLength + sizeof( BITMAPINFOHEADER ) + 1024;
		return FALSE;
	}

	PBYTE	pDib;
	if ( !( pDib = ( PBYTE )::GlobalLock( m_hDib )))
		return FALSE;

	(( BITMAPINFOHEADER* )pDib )->biSize = sizeof( BITMAPINFOHEADER );
	(( BITMAPINFOHEADER* )pDib )->biWidth = m_dwWidth;
	(( BITMAPINFOHEADER* )pDib )->biHeight = m_dwHeight;
	(( BITMAPINFOHEADER* )pDib )->biPlanes = 1;
	(( BITMAPINFOHEADER* )pDib )->biBitCount = 8;
	(( BITMAPINFOHEADER* )pDib )->biCompression = BI_RGB;
	(( BITMAPINFOHEADER* )pDib )->biSizeImage = 0;
	(( BITMAPINFOHEADER* )pDib )->biXPelsPerMeter = 0;
	(( BITMAPINFOHEADER* )pDib )->biYPelsPerMeter = 0;
	(( BITMAPINFOHEADER* )pDib )->biClrUsed = 0;
	(( BITMAPINFOHEADER* )pDib )->biClrImportant = 0;

	::GlobalUnlock( m_hDib );

	pDib = LockBits ();

	for ( int i = 0; i < sizeY; i ++ )
		memcpy ( &pDib  [( sizeY-i-1)*m_dwRowLength], &pBuffer [i*sizeX], sizeX );

	UnlockBits ();

	SetGrayPalette ();

	return TRUE;
}

//****************************************************************************
//	CGrayDib class
//			CreateBuffer()

BYTE* CGrayDib::CreateBuffer()
{
	if ( !m_hDib )
		return NULL;

	PBYTE	pBuffer = ( PBYTE )malloc ( Width ()*Height () );

	if ( !pBuffer )
		return NULL;

	PBYTE	pDib;
	pDib = LockBits ();

	for ( int i = 0; i < Height (); i ++ )
		memcpy ( &( pBuffer [i*Width ()] ), &pDib  [( Height ()-i-1)*m_dwRowLength], Width () );

	UnlockBits ();

	return pBuffer;
}

//****************************************************************************
//	CGrayDib class
//			Fill( BYTE clrColor )

BOOL CGrayDib::Fill( BYTE clrColor )
{
	PBYTE	pBits;
	if ( !( pBits = LockBits()))
		return FALSE;
		
	for ( DWORD j = 0; j < m_dwHeight; j ++ )
		for ( DWORD i = 0; i < m_dwWidth; i ++ )
			pBits[ j * m_dwRowLength + i ] = clrColor;

	UnlockBits();

	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			LockBits()

PBYTE CGrayDib::LockBits()
{
	PBYTE	pDib;
	if ( !( pDib = ( PBYTE )::GlobalLock( m_hDib )))
		return NULL;

	PBYTE	pBits = ( PBYTE )(( PBYTE )pDib + *( PDWORD )pDib ) + 1024;

	return pBits;
}
//****************************************************************************
//	CGrayDib class
//			UnlockBits()

BOOL CGrayDib::UnlockBits()
{
	::GlobalUnlock( m_hDib );

	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			AlignToDword()

DWORD CGrayDib::AlignToDword( DWORD Value )
{
	if ( Value % 4 )
		Value = ( DWORD )( Value / 4 ) * 4 + 4;
	return Value;
}
//****************************************************************************
//	CGrayDib class
//			GetSize()

CSize CGrayDib::GetSize()
{
	CSize	szSize;

	szSize.cx = m_dwWidth;
	szSize.cy = m_dwHeight;

	return szSize;
}
//****************************************************************************
//	CGrayDib class
//			Width()

int CGrayDib::Width()
{
	return ( int )m_dwWidth;
}
//****************************************************************************
//	CGrayDib class
//			Height()

int CGrayDib::Height()
{
	return ( int )m_dwHeight;
}
//****************************************************************************
//	CGrayDib class
//			RowLength()

int CGrayDib::RowLength()
{
	return ( int )m_dwRowLength;
}
//****************************************************************************
//	CGrayDib class
//			Length()

int CGrayDib::Length()
{
	return ( int )m_dwDibLength;
}
//****************************************************************************
//	CGrayDib class
//			Draw()

BOOL CGrayDib::Draw( CDC* pDC, CRect rRect )
{
	LPBITMAPINFOHEADER pbiHeader = (LPBITMAPINFOHEADER)GlobalLock( m_hDib );
	PBYTE pBits = (BYTE*)pbiHeader + pbiHeader->biSize + 1024;
		
	HDRAWDIB hdd = DrawDibOpen();

	if ( hdd )
	{
		DrawDibDraw( hdd, pDC->m_hDC,
					rRect.left, rRect.top, rRect.Width(), rRect.Height(),	
					pbiHeader,	
					pBits,	
					0, 0, -1, -1,	
					0);
		DrawDibClose( hdd );
	} 
	else
	{
		TRACE0( "DrawDibOpen failed\n" );
		return FALSE;
	}
	
	GlobalUnlock( m_hDib );
	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			Stretch()

BOOL CGrayDib::Stretch( CDC* pDC, CRect rRect )
{
	if(m_hDib == NULL)
		return FALSE;
	LPBITMAPINFOHEADER pbiHeader = (LPBITMAPINFOHEADER)GlobalLock( m_hDib );
	PBYTE pBits = (BYTE*)pbiHeader + pbiHeader->biSize + 1024;

	int r = ::StretchDIBits(pDC->m_hDC,
				rRect.left, rRect.top, rRect.Width(), rRect.Height(),
							   0, 0,
							   Width (),
							   Height (),
							   pBits,
							   (LPBITMAPINFO)pbiHeader,
							   DIB_RGB_COLORS,
							   SRCCOPY);

	GlobalUnlock( m_hDib );
	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			Another DrawWithoutStretch()

BOOL CGrayDib::DrawWithoutStretch( CDC* pDC, CPoint Point )
{
	LPBITMAPINFOHEADER pbiHeader = (LPBITMAPINFOHEADER)GlobalLock( m_hDib );
	PBYTE pBits = (BYTE*)pbiHeader + pbiHeader->biSize + 1024;
		
	HDRAWDIB hdd = DrawDibOpen();

	if ( hdd )
	{
		DrawDibDraw( hdd, pDC->m_hDC,
					Point.x, Point.y, Width (), Height (),
					pbiHeader,
					pBits,
					0, 0, Width (), Height (),
					0);
		DrawDibClose( hdd );
	} 
	else
	{
		TRACE0( "DrawDibOpen failed\n" );
		return FALSE;
	}

	GlobalUnlock( m_hDib );
	return TRUE;
}
//****************************************************************************
//	CGrayDib class
//			LoadDIBFromResource ()

BOOL CGrayDib::LoadDibFromResource ( LPCTSTR lpName, LPCTSTR lpType )
{
	HINSTANCE	hInst = AfxGetResourceHandle();
	HRSRC		hRes;
    HGLOBAL		hData, hDib;
	
    if ((hRes = ::FindResource( hInst, lpName,
          lpType ) ) != NULL &&
        (hData = ::LoadResource( hInst, hRes)) != NULL)
    {
		DWORD dwSize = SizeofResource( hInst, hRes );
		LPVOID lpDib = LockResource ( ( HGLOBAL )hData );

		if ( !lpDib )
			return FALSE;

		hDib = GlobalAlloc ( GHND, dwSize );
//		SumMemGA += dwSize;

		if ( !hDib )
			return FALSE;

		memcpy ( GlobalLock ( hDib ), ( LPSTR )lpDib + 14, dwSize - 14 );
		
		GlobalUnlock ( hDib );
		FreeResource(hData);

		if ( m_hDib )
		{
		//	SumMemGA -= GlobalSize( m_hDib );
			GlobalFree ( m_hDib );
		}

		m_hDib = hDib;

		BITMAPINFOHEADER	bi;

		GetBitmapInfoHeader ( &bi );

		m_dwWidth = bi.biWidth;
		m_dwHeight = bi.biHeight;
		m_dwRowLength = AlignToDword ( m_dwWidth );
	
		m_dwDibLength = m_dwRowLength * m_dwHeight;

		return TRUE;
    }
		
	return FALSE;
}
//****************************************************************************
//	CGrayDib class
//			GetBitmapInfoHeader ()

BOOL CGrayDib::GetBitmapInfoHeader ( LPBITMAPINFOHEADER lpbi )
{
	LPBITMAPINFOHEADER *lpsrc;

	if ( m_hDib )
	{
		lpsrc = ( LPBITMAPINFOHEADER * )GlobalLock ( m_hDib );
		memcpy ( lpbi, lpsrc, sizeof ( BITMAPINFOHEADER ) );
		GlobalUnlock ( m_hDib );

		return TRUE;
	}
	else 
		return FALSE;
}

BOOL CGrayDib::SaveDibToBmpFile(CFile& file)
{
	BITMAPFILEHEADER bmfHdr; // Header for Bitmap file
	LPBITMAPINFOHEADER lpBI;   // Pointer to DIB info structure
	DWORD dwDIBSize;

	if (m_hDib == NULL)
		return FALSE;

	lpBI = (LPBITMAPINFOHEADER) ::GlobalLock((HGLOBAL) m_hDib);
	if (lpBI == NULL)
		return FALSE;

	bmfHdr.bfType = 0x4d42;  // "BM"
	dwDIBSize = *(LPDWORD)lpBI + m_dwRowLength*m_dwHeight + 1024;

	// Calculate the file size by adding the DIB size to sizeof(BITMAPFILEHEADER)

	bmfHdr.bfSize = dwDIBSize + sizeof(BITMAPFILEHEADER);
	bmfHdr.bfReserved1 = 0;
	bmfHdr.bfReserved2 = 0;

	bmfHdr.bfOffBits = (DWORD)sizeof(BITMAPFILEHEADER) + lpBI->biSize;
											  
	TRY
	{
		file.Write((LPSTR)&bmfHdr, sizeof(BITMAPFILEHEADER));
		file.WriteHuge(lpBI, dwDIBSize);
	}
	CATCH (CFileException, e)
	{
		::GlobalUnlock((HGLOBAL) m_hDib);
		THROW_LAST();
	}
	END_CATCH

	::GlobalUnlock((HGLOBAL) m_hDib);
	return TRUE;
}

BOOL CGrayDib::LoadDibFromBmpFile(CFile& file)
{
	BITMAPFILEHEADER	bmfHeader;
	DWORD				dwBitsSize;
	HGLOBAL				hDIB;
	LPSTR				pDIB;

	if ( m_hDib )
	{
	//	SumMemGA -= GlobalSize( m_hDib );
		GlobalFree ( m_hDib );
	}

	dwBitsSize = file.GetLength();
	
	if (file.Read((LPSTR)&bmfHeader, sizeof(bmfHeader)) != sizeof(bmfHeader))
		return FALSE;

	if (bmfHeader.bfType != 0x4d42)
		return FALSE;

	hDIB = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, dwBitsSize);
//	SumMemGA += dwBitsSize;

	if (hDIB == 0)
		return FALSE;

	pDIB = (LPSTR) ::GlobalLock(hDIB);

	if (file.ReadHuge(pDIB, dwBitsSize - sizeof(BITMAPFILEHEADER)) !=
		dwBitsSize - sizeof(BITMAPFILEHEADER) )
	{
		::GlobalUnlock(hDIB);
//		SumMemGA -= GlobalSize( hDIB );
		::GlobalFree(hDIB);
		return NULL;
	}

	::GlobalUnlock(hDIB);

	m_hDib = hDIB;

	BITMAPINFOHEADER	bi;

	GetBitmapInfoHeader ( &bi );

	m_dwWidth = bi.biWidth;
	m_dwHeight = bi.biHeight;
	m_dwRowLength = AlignToDword ( m_dwWidth );
	m_dwDibLength = m_dwRowLength * m_dwHeight;

	return TRUE;
}

BOOL CGrayDib::LoadDibFromPcxFile(CFile& file)
{
	BITMAPINFOHEADER bmHeader;

	DWORD		i = 0;
	PCXHeader	Header;
 
	DWORD		PackedImageSize;
	BYTE*		PackedImage;
	BYTE*		PackedBytes;
	DWORD		PackCount;
	DWORD		sizeX, sizeY;

	if ( m_hDib )
	{
	//	SumMemGA -= GlobalSize( m_hDib );
		GlobalFree ( m_hDib );
	}

	file.Read ( &Header, sizeof( PCXHeader ) );

	if ( Header.PlanesNumber != 1 || Header.BitPerPixel != 8 || 
		 Header.ZsoftSign != 10 || Header.Version != 5 ) 
	{
		file.Seek ( 0, CFile::begin );
		return( FALSE );
	}

	sizeX = Header.X2 - Header.X1 + 1;
	sizeY = Header.Y2 - Header.Y1 + 1;

	HGLOBAL	hDIB = ::GlobalAlloc( GHND,  AlignToDword ( sizeX )*sizeY + sizeof( BITMAPINFOHEADER ) + 1024 );
//	SumMemGA += AlignToDword ( sizeX )*sizeY + sizeof( BITMAPINFOHEADER ) + 1024;
	
	if ( !hDIB )
	{
		return FALSE;
	}

	LPSTR	lpDIB = ( LPSTR )::GlobalLock(( HGLOBAL )hDIB );
	LPSTR	lpDIBPalette = lpDIB + sizeof( BITMAPINFOHEADER );
	LPSTR	lpDIBBits = lpDIBPalette + 1024;

	PackedImageSize = file.SeekToEnd () - sizeof( PCXHeader ) - 769;

	if (( PackedImage = ( BYTE* )malloc( PackedImageSize+100)) == NULL )
	{
		GlobalUnlock ( hDIB );
		return FALSE;
	}

	file.Seek ( sizeof( PCXHeader ), CFile::begin );
	file.Read ( PackedImage, PackedImageSize );

	PackedBytes = PackedImage;

	while ( i < (( sizeY ) * ( ( DWORD )Header.BytePerLine )) - 1 )
	{
		if ( *PackedBytes > ( BYTE )191 )
		{
			PackCount = *( PackedBytes ++ ) ^ ( BYTE )0xC0;
			memset( &( lpDIBBits [ i ] ), ( int )*( PackedBytes ++ ), PackCount );
			i += PackCount;
		}
		else
			lpDIBBits [ i ++ ] = *( PackedBytes ++ );
    
	}

	bmHeader.biSize = 40;
	bmHeader.biWidth = Header.X2 - Header.X1 + 1;
	bmHeader.biHeight = Header.Y2 - Header.Y1 + 1;
	bmHeader.biPlanes = 1;
	bmHeader.biBitCount = 8;
	bmHeader.biCompression = BI_RGB;
	bmHeader.biSizeImage = 0;
	bmHeader.biXPelsPerMeter = 0;
	bmHeader.biYPelsPerMeter = 0;
	bmHeader.biClrUsed = 0;
	bmHeader.biClrImportant = 0; 

	m_dwWidth = bmHeader.biWidth;
	m_dwHeight = bmHeader.biHeight;
	m_dwRowLength = AlignToDword ( m_dwWidth );
	m_dwDibLength = m_dwRowLength * m_dwHeight;

	BYTE	Palette[ 1024 ];
	int		j;

	for ( i = 0, j = 0; i < 256; i ++, j += 4 )
	{
		Palette[ j ] = ( BYTE )i;
		Palette[ j + 1 ] = ( BYTE )i;
		Palette[ j + 2 ] = ( BYTE )i;
		Palette[ j + 3 ] = 0;
	}

	memcpy( lpDIB, &bmHeader, bmHeader.biSize );
	memcpy( lpDIBPalette, Palette, 1024 );

	::GlobalUnlock( hDIB );
	free( PackedImage );

	m_hDib = hDIB;

	Revert ();
	
	return TRUE;
}

BOOL CGrayDib::LoadDibFromBmpFile( CString strFileName )
{
	CFile file;
	CFileException e;

	if ( !file.Open ( strFileName, CFile::modeRead, &e ) )
		return FALSE;

	int nRetCode = LoadDibFromBmpFile ( file );

	file.Close ();

	return nRetCode;
}

BOOL CGrayDib::LoadDibFromPcxFile( CString strFileName )
{
	CFile file;
	CFileException e;

	if ( !file.Open ( strFileName, CFile::modeRead, &e ) )
		return FALSE;

	int nRetCode = LoadDibFromPcxFile ( file );

	file.Close ();

	return nRetCode;
}

BOOL CGrayDib::SaveDibToBmpFile( CString strFileName )
{
	CFile file;
	CFileException e;

	if ( !file.Open ( strFileName, CFile::modeCreate | 
		CFile::modeWrite, &e ) )
		return FALSE;

	int nRetCode = SaveDibToBmpFile ( file );

	file.Close ();

	return nRetCode;
}

void CGrayDib::SetGrayPalette ()
{
	PBYTE pBits;

	if ( !m_hDib || !( pBits = LockBits () ) )
		return;

	pBits -= 1024;

	for ( int i = 0; i < 256; i ++ )
	{
		pBits [i*4] = i;
		pBits [i*4+1] = i;
		pBits [i*4+2] = i;
	}

	UnlockBits ();
}

void CGrayDib::InvertPalette ()
{
	PBYTE pBits;

	if ( !m_hDib || !( pBits = LockBits () ) )
		return;

	pBits -= 1024;

	for ( int i = 0; i < 256; i ++ )
	{
		pBits [i*4] = 255-pBits [i*4];
		pBits [i*4+1] = 255-pBits [i*4+1];
		pBits [i*4+2] = 255-pBits [i*4+2];
	}

	UnlockBits ();
}

BOOL CGrayDib::IsValidGrayDib ()
{
	if ( !m_hDib )
		return FALSE;

	BITMAPINFOHEADER	bi;

	GetBitmapInfoHeader ( &bi );

	if ( bi.biPlanes == 1 && bi.biBitCount == 8 )
		return TRUE;
	else
		return FALSE;
}

BOOL CGrayDib::Paste( CGrayDib* pdPasteDib, CPoint ptPoint )
{
	PBYTE	pbBits = LockBits();
	PBYTE	pbPasteBits = pdPasteDib->LockBits();

	CRect	rRect( ptPoint, CSize( pdPasteDib->Width(), pdPasteDib->Height()));

	if ( rRect.left >= 0 && rRect.top >= 0 &&
		rRect.right < Width() && rRect.bottom < Height())
	{
		for ( int i = 0; i < rRect.Height(); i ++ )
		{
			memcpy( &pbBits[( Height() - i - 1 - rRect.top ) * RowLength() + rRect.left ],
					&pbPasteBits[( pdPasteDib->Height() - i - 1 )  * pdPasteDib->RowLength() ],
					rRect.Width() );
		}
	}
	else
	{
		for ( int i = rRect.top; i < rRect.bottom; i ++ )
		{
			for ( int j = rRect.left; j < rRect.right; j ++ )
			{
				if ( i >= 0 && j >= 0 && i < Height() && j < Width())
				{
					pbBits[( Height() - i - 1 ) * RowLength() + j] = 
					pbPasteBits[( pdPasteDib->Height() - ( i - rRect.top ) - 1 ) * pdPasteDib->RowLength() + ( j - rRect.left )];
				}
			}
		}

	}

	pdPasteDib->UnlockBits();
	UnlockBits();

	return TRUE;
}

BYTE CGrayDib::CalculateMiddle()
{
	ASSERT ( m_hDib );

	if ( !m_hDib )
		return 0;

	DWORD nMiddle = 0;

	PBYTE	pBits = LockBits();

	for ( DWORD j = 0; j < m_dwHeight; j ++ )
		for ( DWORD i = 0; i < m_dwWidth; i ++ )
			nMiddle += ( DWORD )( pBits[ j * m_dwRowLength + i ] );

	nMiddle /= m_dwWidth*m_dwHeight;

	UnlockBits ();

	return ( BYTE )nMiddle;
}

BOOL CGrayDib::Center ( BYTE bDestMiddle )
{
	ASSERT ( m_hDib );

	if ( !m_hDib )
		return FALSE;

	BYTE bMiddle = CalculateMiddle ();
	int nDelta = ( int )bDestMiddle - ( int )bMiddle;

	PBYTE	pBits = LockBits();

	for ( DWORD j = 0; j < m_dwHeight; j ++ )
		for ( DWORD i = 0; i < m_dwWidth; i ++ )
		{
			if ( nDelta > 0 )
				pBits[ j * m_dwRowLength + i ] += ( BYTE )nDelta;
			else
				pBits[ j * m_dwRowLength + i ] -= ( BYTE )( -nDelta );
		}

	UnlockBits ();

	return TRUE;
}


//****************************************************************************
//	CGrayDib class
//			ZoomIn()

CGrayDib* CGrayDib::ZoomIn( CSize szSize )
{
	PBYTE	pBits = LockBits();
	CGrayDib*	pdZoom = new CGrayDib;

	pdZoom->Create( szSize );
	PBYTE	pZoom = pdZoom->LockBits();
	
	int		i,j,k,l;

	DWORD	SumB;
	DWORD	count;
	DWORD	row = Height();
	DWORD	pix = Width();
	DWORD	new_row = szSize.cy;
	DWORD	new_pix = szSize.cx;

	double	x_rat = (double)pix / (double)new_pix;
	double	y_rat = (double)row / (double)new_row;

	if (( x_rat < 1 ) || ( y_rat < 1 ))
		return NULL;

	for( i = 0; i < (int)new_row; i++ )
	{
		for( j = 0; j < (int)new_pix; j ++ )
		{
			SumB = 0;
			count = 0;

			for( k =(int)(i*y_rat); k<(int)((i+1)*y_rat); k++ )
			{	
				for(l=(int)(j*x_rat); l < (int)((j+1)*x_rat); l++ )
				{
					SumB = SumB + (BYTE)pBits[ k * AlignToDword(pix) + l];
					count ++;
				}
			}

			SumB = SumB / count;

			if ( SumB > 255 ) SumB = 255;
				
			pZoom[ i * AlignToDword(new_pix) + j] = (BYTE)SumB;
		}
	}

	UnlockBits();
	pdZoom->UnlockBits();

	return pdZoom;
}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//****************************************************************************
//	CGrayDib class
//			ZoomX_Out ()

CGrayDib *CGrayDib::ZoomX_Out( DWORD	new_pix )
{
	DWORD			row = Height ();
	DWORD			pix = Width ();
	DWORD			new_row = row;

	if( new_pix < pix )
		return NULL;

	CGrayDib			*pdbDest = new CGrayDib;

	if ( !pdbDest )
		return NULL;

	if ( !pdbDest -> Create ( CSize ( new_pix, new_row ) ) )
		return NULL;

	BYTE *lpDIB = LockBits ();
	BYTE *lpNEW = pdbDest -> LockBits ();

	int		xn, xn1;
	int		i, j;
	DWORD	Rn, Rn1;
	int		R;
	double	x_rat;

	x_rat = (double)(pix-1) / (double)(new_pix-1); // x_rat < 1

	for( i = 0; i < (int)new_row; i++ )
	{
		for( j = 0; j < (int)new_pix; j ++ )
		{
			xn  = (int)( j * x_rat );

			if( j == (int)(new_pix - 1) ) 
				xn1 = (int)( j * x_rat );
			else 
				xn1 = (int)( j * x_rat ) + 1;

			
			Rn	= (BYTE)lpDIB[ i*m_dwRowLength + xn ];
			Rn1	= (BYTE)lpDIB[ i*m_dwRowLength + xn1];

			if( Rn1 >= Rn )
				R = (int)(Rn1-(float)(Rn1-Rn)*((int)(j*x_rat)-(j*x_rat)));
			else
				R = (int)(Rn -(float)(Rn-Rn1)*((j*x_rat)-(int)(j*x_rat)));

			if( R < 0 )		R = 0;
			if( R > 255 )	R = 255;

			lpNEW[ i*pdbDest -> m_dwRowLength + j ]	 = (BYTE)R; 
		}
	}

	UnlockBits ();
	pdbDest -> UnlockBits ();

	return pdbDest;
}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//****************************************************************************
//	CGrayDib class
//			ZoomY_Out ()

CGrayDib *CGrayDib::ZoomY_Out( DWORD new_row )
{
	DWORD			row = Height ();
	DWORD			pix = Width ();
	DWORD			new_pix = pix;

	if( new_row < row )
		return NULL;

	CGrayDib *pdbDest = new CGrayDib;

	if ( !pdbDest )
		return NULL;

	if ( !pdbDest -> Create ( CSize ( new_pix, new_row ) ) )
		return NULL;

	BYTE *lpDIB = LockBits ();
	BYTE *lpNEW = pdbDest -> LockBits ();

	double	y_rat;
	int		yn, yn1;
	int		i, j;
	int		R;
	DWORD	Rn, Rn1;

	y_rat = (double)(row-1) / (double)(new_row-1); // y_rat < 1

	for( i = 0; i < (int)new_row; i++ )
	{
		yn  = (int)( i * y_rat );

		if( i == (int)(new_row - 1) ) 
			yn1 = (int)( i * y_rat );
		else 
			yn1 = (int)( i * y_rat ) + 1;

		for( j = 0; j < (int)pix; j ++ )
		{
			Rn	= (BYTE)lpDIB[ yn * m_dwRowLength + j ];
			Rn1	= (BYTE)lpDIB[ yn1* m_dwRowLength + j ];

			if( Rn1 >= Rn )
				R = (int)(Rn1-(float)(Rn1-Rn)*((int)(i*y_rat)-(i*y_rat)));
			else
				R = (int)(Rn -(float)(Rn-Rn1)*((i*y_rat)-(int)(i*y_rat)));

			if( R < 0 )		R = 0;
			if( R > 255 )	R = 255;

			lpNEW[ i*pdbDest -> m_dwRowLength + j]	 = (BYTE)R; 
		}
	}

	UnlockBits ();
	pdbDest -> UnlockBits ();

	return pdbDest;
}
//****************************************************************************
//	CGrayDib class
//			Zoom ()

CGrayDib *CGrayDib::Zoom ( CSize szNewSize )
{
	CGrayDib	*dbTmp, *dbDest;

	if ( szNewSize.cx <= ( int )Width () && szNewSize.cy <= ( int )Height () )
		return ZoomIn ( szNewSize );

	if ( szNewSize.cx > ( int )Width () )
		dbTmp = ZoomX_Out ( ( DWORD )szNewSize.cx );
	else
	if ( szNewSize.cx == ( int )Width () )
		dbTmp = this;
	else
		dbTmp = ZoomIn ( CSize ( szNewSize.cx, Height () ) );


	if ( szNewSize.cy > ( int )Height () )
		dbDest = dbTmp -> ZoomY_Out ( ( DWORD )szNewSize.cy );
	else
	if ( szNewSize.cy == ( int )Height () )
		return dbTmp;
	else
		dbDest = dbTmp -> ZoomIn ( CSize ( dbTmp -> Width (), szNewSize.cy ) );

	if ( dbTmp && dbTmp != this )
		delete dbTmp;

	return dbDest;
}

//****************************************************************************
//	End of GrayDib.cpp implementation file.

